from argparse import ArgumentParser
from os.path import isfile, join, splitext, basename
from os import listdir
from multiprocessing import Process
from lotte.Lotte import Lotte
import time
import json
from pathlib import Path
from datetime import datetime


def json_encoder(obj):
    return obj.__dict__


def process_file(literature_content, scholarly_file_path, output_folder_path):
    filename = splitext(basename(scholarly_file_path))[0]
    output_file = open(join(output_folder_path, filename + '.json'), 'w')

    with open(scholarly_file_path, 'r') as scholar_file:
        scholarly_content = scholar_file.read()

    lotte = Lotte(5)
    matches = lotte.compare(literature_content, scholarly_content)
    json_string = json.dumps(matches, default=json_encoder)
    print_line(json_string, output_file)
    output_file.close()


def print_line(line, file):
    file.write(line)


def main():
    argument_parser = ArgumentParser()

    argument_parser.add_argument("-l", "--literature_path", dest="literature_file_path",
                                 help="Der Pfad zur Literaturdatei", required=True)
    argument_parser.add_argument("-s", "--scholarly_path", dest="scholarly_folder_path",
                                 help="Der Pfad zum Ordner mit den Aufsätzen", required=True)
    argument_parser.add_argument("-o", "--output_path", dest="output_folder_path",
                                 help="Der Pfad zum Ausgabeordner", required=True)

    args = argument_parser.parse_args()

    literature_file_path = args.literature_file_path
    scholarly_file_or_folder_path = args.scholarly_folder_path

    now = datetime.now()
    date_time_string = now.strftime("%Y_%m_%d_%H_%M_%S")
    output_folder_path = join(args.output_folder_path, date_time_string)
    Path(output_folder_path).mkdir(parents=True, exist_ok=True)

    with open(literature_file_path, 'r') as literature_file:
        literature_content = literature_file.read()

    start_time = time.time()

    if isfile(scholarly_file_or_folder_path) and scholarly_file_or_folder_path.endswith(".txt"):
        process_file(literature_content, scholarly_file_or_folder_path, output_folder_path)
    else:
        processes = []

        for file_or_folder in listdir(scholarly_file_or_folder_path):
            full_path = join(scholarly_file_or_folder_path, file_or_folder)

            if isfile(full_path) and full_path.endswith(".txt"):
                process = Process(target=process_file, args=(literature_content, full_path, output_folder_path))
                processes.append(process)
                process.start()

        for process in processes:
            process.join()

    print("\n\n--- %s seconds ---" % (time.time() - start_time))


if __name__ == '__main__':
    main()
