from argparse import ArgumentParser
from os.path import isfile, join
from os import listdir
from lotte.Lotte import Lotte


def process_file(scientific_file, literature_content):

    scientific_content = open(scientific_file, "r").read()
    lotte = Lotte(5)
    matches = lotte.compare(literature_content, scientific_content)
    result = ''

    for match in matches:
        match_literature = match.source_match_segment
        match_scientific = match.target_match_segment

        content = literature_content[match_literature.character_start_pos:match_literature.character_end_pos]
        result += f'\n{match_literature.character_start_pos}\t{match_literature.character_end_pos}\t{content}'

        content = scientific_content[match_scientific.character_start_pos:match_scientific.character_end_pos]
        result += f'\n{match_scientific.character_start_pos}\t{match_scientific.character_end_pos}\t{content}'

    print(result)


def main():
    argument_parser = ArgumentParser()

    argument_parser.add_argument("-l", "--literature_path", dest="literature_file_path",
                                 help="Der Pfad zur Literaturdatei", required=True)
    argument_parser.add_argument("-s", "--scholarly_path", dest="scholarly_folder_path",
                                 help="Der Pfad zum Ordner mit den Aufsätzen", required=True)

    args = argument_parser.parse_args()

    literature_file_path = args.literature_file_path
    scholarly_folder_path = args.scholarly_folder_path
    literature_content = open(literature_file_path, "r").read()

    if isfile(scholarly_folder_path) and scholarly_folder_path.endswith(".txt"):
        process_file(scholarly_folder_path, literature_content)
    else:
        for fileOrFolder in listdir(scholarly_folder_path):
            full_path = join(scholarly_folder_path, fileOrFolder)

            if isfile(full_path) and full_path.endswith(".txt"):
                process_file(full_path, literature_content)


if __name__ == '__main__':
    main()
