import unittest
from lotte.Lotte import Lotte
from test.Helper import get_text


class LotteTestCase(unittest.TestCase):

    def test_max_merge_ellipse_distance(self):
        source_text = 'This is a long Text but there are too many words here are even more words here to test.'
        target_text = 'This is a long Text ... even more words here to test.'

        lotte = Lotte(5, look_back_limit=2, max_merge_ellipse_distance=3)
        matches = lotte.compare(source_text, target_text)

        self.assertEqual(2, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(19, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(19, matches[0].target_match_segment.character_end_pos)

        self.assertEqual(58, matches[1].source_match_segment.character_start_pos)
        self.assertEqual(87, matches[1].source_match_segment.character_end_pos)
        self.assertEqual(24, matches[1].target_match_segment.character_start_pos)
        self.assertEqual(53, matches[1].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text',
                         get_text(target_text, matches[0].target_match_segment))

        lotte = Lotte(5, look_back_limit=2, max_merge_ellipse_distance=10)
        matches = lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(87, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(53, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text but there are too many words here are even more words here to test.',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text ... even more words here to test.',
                         get_text(target_text, matches[0].target_match_segment))

    def test_max_merge_distance(self):
        source_text = 'This is a long Text okay why do even more words here to test.'
        target_text = 'This is a long Text some not fitting even more words here to test.'

        lotte = Lotte(5, look_back_limit=2, max_merge_distance=2)
        matches = lotte.compare(source_text, target_text)

        self.assertEqual(2, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(19, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(19, matches[0].target_match_segment.character_end_pos)

        self.assertEqual(32, matches[1].source_match_segment.character_start_pos)
        self.assertEqual(61, matches[1].source_match_segment.character_end_pos)
        self.assertEqual(37, matches[1].target_match_segment.character_start_pos)
        self.assertEqual(66, matches[1].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text',
                         get_text(target_text, matches[0].target_match_segment))

        self.assertEqual('even more words here to test.',
                         get_text(source_text, matches[1].source_match_segment))
        self.assertEqual('even more words here to test.',
                         get_text(target_text, matches[1].target_match_segment))

        lotte = Lotte(5, look_back_limit=2, max_merge_distance=10)
        matches = lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(61, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(66, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text okay why do even more words here to test.',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text some not fitting even more words here to test.',
                         get_text(target_text, matches[0].target_match_segment))
