import unittest
from lotte.Lotte import Lotte
from test.Helper import get_text


class LotteTestCase(unittest.TestCase):

    def setUp(self):
        self.lotte = Lotte(5)

    def test_full(self):
        source_text = 'This is a long Text and the long test goes on and on and on'
        target_text = 'This is [...] and the long test goes on'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(45, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(39, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text and the long test goes on',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is [...] and the long test goes on',
                         get_text(target_text, matches[0].target_match_segment))

    def test_sentence_boundary_1(self):
        source_text = 'This is a long Text and the long test goes on. And continues'
        target_text = 'This is [...] and the long test goes on. And continues'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(46, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(40, matches[0].target_match_segment.character_end_pos)

    def test_sentence_boundary_2(self):
        source_text = 'This is a long Text and the long test goes on. And continues!'
        target_text = 'This is [...] and the long test goes on. And continues!'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(61, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(55, matches[0].target_match_segment.character_end_pos)

    def test_quoted_twice(self):
        source_text = 'This is a long Text and the long test goes on.\n\n'
        target_text = 'This is a long Text and the long test goes on.\n\n' \
                      'And now some other Text. And now is a long Text and the long test goes on again'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(2, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(46, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(46, matches[0].target_match_segment.character_end_pos)

        self.assertEqual(5, matches[1].source_match_segment.character_start_pos)
        self.assertEqual(46, matches[1].source_match_segment.character_end_pos)
        self.assertEqual(81, matches[1].target_match_segment.character_start_pos)
        self.assertEqual(121, matches[1].target_match_segment.character_end_pos)

    def test_backward_extension(self):
        source_text = 'This is a long Text and the long test goes on.\n\n'
        target_text = 'This is [...] and the long test goes on.\n\n'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(46, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(40, matches[0].target_match_segment.character_end_pos)

    def test_similar_quotations(self):
        source_text = 'This is a long Text and the long test goes on.\n\n' \
                      'This a long Text and the long test doesn\'t go on'
        target_text = 'This is a long Text the long test goes on.\n\n'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(46, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(42, matches[0].target_match_segment.character_end_pos)

    def test_fuzzy_matching_1(self):
        source_text = 'This is a long Text and the long test goes on.'
        target_text = 'This is a long Teext the long test goes on.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))
        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(46, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(43, matches[0].target_match_segment.character_end_pos)

    def test_missing_word(self):
        source_text = 'This is a long Text and the test goes on.'
        target_text = 'This is a long Text and the long test goes on.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(41, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(46, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text and the test goes on.',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text and the long test goes on.',
                         get_text(target_text, matches[0].target_match_segment))

    def test_overlapping_target_matches(self):
        source_text = 'And the long test goes. And the long test goes on.'
        target_text = 'This is a long Text and the long test goes on.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(24, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(50, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(20, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(46, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('And the long test goes on.',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('and the long test goes on.',
                         get_text(target_text, matches[0].target_match_segment))

    def test_look_back(self):
        source_text = 'A long Text but so on and on.'
        target_text = 'A long Text but so bla blub bleh bloh blieh bluh on and on.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(18, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(18, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('A long Text but so',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('A long Text but so',
                         get_text(target_text, matches[0].target_match_segment))

    def test_merging_matches_1(self):
        source_text = 'This is a long Text but there are too many words here.'
        target_text = 'This is a long Text but too many words here.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(54, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(44, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text but there are too many words here.',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text but too many words here.',
                         get_text(target_text, matches[0].target_match_segment))

    def test_merging_matches_2(self):
        source_text = 'This is a long Text but there are too many words here and then there are ' \
                      'even more words too many.'
        target_text = 'This is a long Text but too many words here. There are even more words.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(88, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(71, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text but there are too many words here and then there are '
                         'even more words',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text but too many words here. There are even more words.',
                         get_text(target_text, matches[0].target_match_segment))

    def test_merging_matches_3(self):
        source_text = 'A long Text but so on and on.\n\n' \
                      'This is a long Text but there are too many words here and then there are ' \
                      'even more words too many. Now I need some text in between' \

        target_text = 'This is a long Text but too many words here.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(31, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(84, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(44, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text but there are too many words here',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text but too many words here.',
                         get_text(target_text, matches[0].target_match_segment))

    @unittest.expectedFailure
    def test_merging_matches_4(self):
        """
        A not working edge case. Shouldn't be important in real world scenarios.
        """

        source_text = 'A long Text but too so on and on.\n\n' \
                      'This is a long Text but there are too many words here and then there are ' \
                      'even more words too many. Now I need some text in between' \

        target_text = 'This is a long Text but too many words here.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(88, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(71, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text but there are too many words here',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text but too many words here.',
                         get_text(target_text, matches[0].target_match_segment))

    def test_merging_matches_5(self):
        source_text = 'A long Text but so on and on and on and on.'
        target_text = 'A long Text but so bla blub bleh bloh blieh bluh on and on and on and on.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(2, len(matches))

        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(18, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(18, matches[0].target_match_segment.character_end_pos)

        self.assertEqual(19, matches[1].source_match_segment.character_start_pos)
        self.assertEqual(43, matches[1].source_match_segment.character_end_pos)
        self.assertEqual(49, matches[1].target_match_segment.character_start_pos)
        self.assertEqual(73, matches[1].target_match_segment.character_end_pos)

        self.assertEqual('A long Text but so',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('A long Text but so',
                         get_text(target_text, matches[0].target_match_segment))
        self.assertEqual('on and on and on and on.',
                         get_text(source_text, matches[1].source_match_segment))
        self.assertEqual('on and on and on and on.',
                         get_text(target_text, matches[1].target_match_segment))

    def test_too_short_matches(self):
        source_text = 'This is a long Text but there are too many words here.'
        target_text = 'This is a long'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(0, len(matches))

    def test_ellipse_1(self):
        source_text = 'This is a long Text but there are too many words here and then there are even more words.'
        target_text = 'This is a long Text ... even more words.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(19, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(19, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text',
                         get_text(target_text, matches[0].target_match_segment))

    def test_ellipse_2(self):
        source_text = 'This is a long Text but there are too many words here are even more words.'
        target_text = 'This is a long Text ... even more words.'

        matches = self.lotte.compare(source_text, target_text)

        self.assertEqual(1, len(matches))

        self.assertEqual(0, matches[0].source_match_segment.character_start_pos)
        self.assertEqual(74, matches[0].source_match_segment.character_end_pos)
        self.assertEqual(0, matches[0].target_match_segment.character_start_pos)
        self.assertEqual(40, matches[0].target_match_segment.character_end_pos)

        self.assertEqual('This is a long Text but there are too many words here are even more words.',
                         get_text(source_text, matches[0].source_match_segment))
        self.assertEqual('This is a long Text ... even more words.',
                         get_text(target_text, matches[0].target_match_segment))


if __name__ == '__main__':
    unittest.main()
