import unittest

from lotte.Lotte import Lotte


class SetupTestCase(unittest.TestCase):

    def test_setup_working(self):
        try:
            Lotte(5)
        except ValueError:
            self.fail("ValueError raised unexpectedly!")

    def test_setup_incorrect_min_match_length(self):
        with self.assertRaises(Exception) as context:
            Lotte(2)
        self.assertEqual('min match length must be >= 3', context.exception.args[0])

    def test_setup_incorrect_look_back_limit(self):
        with self.assertRaises(Exception) as context:
            Lotte(3, look_back_limit=-1)
        self.assertEqual('look back limit must be positive', context.exception.args[0])

    def test_setup_incorrect_look_ahead_limit(self):
        with self.assertRaises(Exception) as context:
            Lotte(3, look_ahead_limit=-1)
        self.assertEqual('look ahead limit must be positive', context.exception.args[0])

    def test_setup_incorrect_max_merge_distance(self):
        with self.assertRaises(Exception) as context:
            Lotte(3, max_merge_distance=-1)
        self.assertEqual('max merge distance must be positive', context.exception.args[0])

    def test_setup_incorrect_max_merge_ellipse_distance(self):
        with self.assertRaises(Exception) as context:
            Lotte(3, max_merge_ellipse_distance=-1)
        self.assertEqual('max merge ellipse distance must be positive', context.exception.args[0])

    def test_empty_source_input(self):
        lotte = Lotte(5)
        matches = lotte.compare('', 'test')

        self.assertEqual(0, len(matches))

    def test_empty_target_input(self):
        lotte = Lotte(5)
        matches = lotte.compare('test', '')

        self.assertEqual(0, len(matches))


if __name__ == '__main__':
    unittest.main()
