from argparse import ArgumentParser
import json
from os.path import join
import re

from visualization.CitationSource import CitationSource
from visualization.SourceSegment import SourceSegment


def json_decoder(json_input):
    if 'source_segment_ids' in json_input:
        return None

    if 'frequency' in json_input:
        return SourceSegment(json_input['my_id'], json_input['start'], json_input['end'], json_input['frequency'],
                             json_input['token_length'], '')
    else:
        return CitationSource(json_input['my_id'], json_input['source_segments'])


def main():
    argument_parser = ArgumentParser()

    argument_parser.add_argument("-l", "--literature_path", dest="literature_file_path",
                                 help="Der Pfad zur Literaturdatei", required=True)
    argument_parser.add_argument("-o", "--output_path", dest="output_folder_path",
                                 help="Der Pfad zum Ausgabeordner", required=True)
    argument_parser.add_argument("-i", "--input", dest="input_folder_path",
                                 help="Der Pfad zum Ordner mit den Ähnlichkeitsdateien", required=True)

    args = argument_parser.parse_args()

    source_file_path = args.literature_file_path
    output_folder_path = args.output_folder_path
    input_folder_path = args.input_folder_path

    with open(source_file_path, 'r') as source_file:
        source_content = source_file.read()

    with open(join(input_folder_path, 'citation_sources.json'), 'r') as citation_sources_file:
        citation_sources = json.load(citation_sources_file, object_hook=json_decoder)

    content = ''
    citation_source_start_pos = 0
    segments = []

    result = ''

    for char_pos in range(0, len(source_content)):
        finished = False
        for citation_source_pos in range(citation_source_start_pos, len(citation_sources)):
            citation_source = citation_sources[citation_source_pos]

            for segment_pos in range(0, len(citation_source.source_segments)):
                segment = citation_source.source_segments[segment_pos]

                if char_pos < segment.start:
                    finished = True
                    break

                if segment.start == char_pos:
                    citation_source_start_pos = citation_source_pos

                    if segment_pos == 0:
                        if content:
                            result += f'<q0>{content}</q0>'
                        segments.clear()
                    else:
                        segments.append((0, content))

                    content = ''
                    finished = True
                    break
                elif segment.end == char_pos:
                    segments.append((segment.frequency, content))
                    content = ''
                    finished = True

                    if segment_pos == len(citation_source.source_segments) - 1:
                        for segment in segments:
                            if segment[1]:
                                if segment[0] == 0 or segment[0] == 1:
                                    result += f'<q{segment[0]}>{segment[1]}</q{segment[0]}>'
                                else:
                                    result += segment[1]
                    break

            if finished:
                break

        content += source_content[char_pos]
    if content:
        result += f'<q0>{content}</q0>'

    print(result)


if __name__ == '__main__':
    main()
